@@ -26,6 +26,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "q_ctype.h"
 #include <errno.h>
 
+#include "miniz.h"
+
 static char	*largv[MAX_NUM_ARGVS + 1];
 static char	argvdummy[] = " ";
 
@@ -295,7 +297,7 @@ void Q_memset (void *dest, int fill, size_t count)
 {
 	size_t		i;
 
-	if ( (((size_t)dest | count) & 3) == 0)
+	if ( (((uintptr_t)dest | count) & 3) == 0)
 	{
 		count >>= 2;
 		fill = fill | (fill<<8) | (fill<<16) | (fill<<24);
@@ -311,7 +313,7 @@ void Q_memcpy (void *dest, const void *src, size_t count)
 {
 	size_t		i;
 
-	if (( ( (size_t)dest | (size_t)src | count) & 3) == 0 )
+	if (( ( (uintptr_t)dest | (uintptr_t)src | count) & 3) == 0)
 	{
 		count >>= 2;
 		for (i = 0; i < count; i++)
@@ -1373,17 +1375,6 @@ void COM_InitArgv (int argc, char **argv)
 	}
 }
 
-/*
-================
-Test_f -- johnfitz
-================
-*/
-#ifdef _DEBUG
-static void FitzTest_f (void)
-{
-}
-#endif
-
 /*
 ================
 COM_Init
@@ -1432,9 +1423,6 @@ void COM_Init (void)
 
 	if (COM_CheckParm("-fitz"))
 		fitzmode = true;
-#ifdef _DEBUG
-	Cmd_AddCommand ("fitztest", FitzTest_f); //johnfitz
-#endif
 }
 
 
@@ -1690,10 +1678,10 @@ static int COM_FindFile (const char *filename, int *handle, FILE **file,
 	if (strcmp(COM_FileGetExtension(filename), "pcx") != 0
 		&& strcmp(COM_FileGetExtension(filename), "tga") != 0
 		&& strcmp(COM_FileGetExtension(filename), "lit") != 0
+		&& strcmp(COM_FileGetExtension(filename), "vis") != 0
 		&& strcmp(COM_FileGetExtension(filename), "ent") != 0)
 		Con_DPrintf ("FindFile: can't find %s\n", filename);
 	else	Con_DPrintf2("FindFile: can't find %s\n", filename);
-		// Log pcx, tga, lit, ent misses only if (developer.value >= 2)
 
 	if (handle)
 		*handle = -1;
@@ -2086,7 +2074,6 @@ _add_path:
 //==============================================================================
 //johnfitz -- dynamic gamedir stuff -- modified by QuakeSpasm team.
 //==============================================================================
-void ExtraMaps_NewGame (void);
 static void COM_Game_f (void)
 {
 	if (Cmd_Argc() > 1)
@@ -2201,6 +2188,7 @@ static void COM_Game_f (void)
 		//clear out and reload appropriate data
 		Cache_Flush ();
 		Mod_ResetAll();
+		Sky_ClearAll();
 		if (!isDedicated)
 		{
 			TexMgr_NewGame ();
@@ -2262,7 +2250,6 @@ void COM_InitFilesystem (void) //johnfitz -- modified based on topaz's tutorial
 	if (COM_CheckParm ("-quoth"))
 		COM_AddGameDirectory (com_basedir, "quoth");
 
-
 	i = COM_CheckParm ("-game");
 	if (i && i < com_argc-1)
 	{
@@ -2272,8 +2259,8 @@ void COM_InitFilesystem (void) //johnfitz -- modified based on topaz's tutorial
 		com_modified = true;
 		// don't load mission packs twice
 		if (COM_CheckParm ("-rogue") && !q_strcasecmp(p, "rogue")) p = NULL;
-		if (COM_CheckParm ("-hipnotic") && !q_strcasecmp(p, "hipnotic")) p = NULL;
-		if (COM_CheckParm ("-quoth") && !q_strcasecmp(p, "quoth")) p = NULL;
+		if (p && COM_CheckParm ("-hipnotic") && !q_strcasecmp(p, "hipnotic")) p = NULL;
+		if (p && COM_CheckParm ("-quoth") && !q_strcasecmp(p, "quoth")) p = NULL;
 		if (p != NULL) {
 			COM_AddGameDirectory (com_basedir, p);
 			// QuakeSpasm extension: treat '-game missionpack' as '-missionpack'
@@ -2461,3 +2448,482 @@ long FS_filelength (fshandle_t *fh)
 	return fh->length;
 }
 
+/*
+============================================================================
+								LOCALIZATION
+============================================================================
+*/
+typedef struct
+{
+	char *key;
+	char *value;
+} locentry_t;
+
+typedef struct
+{
+	int			numentries;
+	int			maxnumentries;
+	int			numindices;
+	unsigned	*indices;
+	locentry_t	*entries;
+	char		*text;
+} localization_t;
+
+static localization_t localization;
+
+/*
+================
+COM_HashString
+Computes the FNV-1a hash of string str
+================
+*/
+unsigned COM_HashString (const char *str)
+{
+	unsigned hash = 0x811c9dc5u;
+	while (*str)
+	{
+		hash ^= *str++;
+		hash *= 0x01000193u;
+	}
+	return hash;
+}
+
+static size_t mz_zip_file_read_func(void *opaque, mz_uint64 ofs, void *buf, size_t n)
+{
+	if (SDL_RWseek((SDL_RWops*)opaque, (Sint64)ofs, RW_SEEK_SET) < 0)
+		return 0;
+	#ifdef USE_SDL2
+	return SDL_RWread((SDL_RWops*)opaque, buf, 1, n);
+	#else
+	else {
+		int r = SDL_RWread((SDL_RWops*)opaque, buf, 1, n);
+		return (r < 0)? 0 : r;
+	}
+	#endif
+}
+
+#ifndef USE_SDL2 /* no SDL_RWsize in SDL-1.2 */
+static Sint32 SDLCALL SDL_RWsize(SDL_RWops *rw) {
+	Sint32 pos, size;
+	if ((pos=SDL_RWtell(rw))<0) return -1;
+	size = SDL_RWseek(rw, 0, RW_SEEK_END);
+	SDL_RWseek(rw, pos, RW_SEEK_SET);
+	return size;
+}
+#endif
+
+/*
+================
+LOC_LoadFile
+================
+*/
+void LOC_LoadFile (const char *file)
+{
+	char path[1024];
+	int i,lineno;
+	char *cursor;
+
+	SDL_RWops *rw = NULL;
+	Sint64 sz;
+	mz_zip_archive archive;
+	size_t size = 0;
+
+	// clear existing data
+	if (localization.text)
+	{
+		free(localization.text);
+		localization.text = NULL;
+	}
+	localization.numentries = 0;
+	localization.numindices = 0;
+
+	if (!file || !*file)
+		return;
+
+	Con_Printf("\nLanguage initialization\n");
+
+	memset(&archive, 0, sizeof(archive));
+	q_snprintf(path, sizeof(path), "%s/%s", com_basedir, file);
+	rw = SDL_RWFromFile(path, "rb");
+	#if defined(DO_USERDIRS)
+	if (!rw) {
+		q_snprintf(path, sizeof(path), "%s/%s", host_parms->userdir, file);
+		rw = SDL_RWFromFile(path, "rb");
+	}
+	#endif
+	if (!rw)
+	{
+		q_snprintf(path, sizeof(path), "%s/QuakeEX.kpf", com_basedir);
+		rw = SDL_RWFromFile(path, "rb");
+		#if defined(DO_USERDIRS)
+		if (!rw) {
+			q_snprintf(path, sizeof(path), "%s/QuakeEX.kpf", host_parms->userdir);
+			rw = SDL_RWFromFile(path, "rb");
+		}
+		#endif
+		if (!rw) goto fail;
+		sz = SDL_RWsize(rw);
+		if (sz <= 0) goto fail;
+		archive.m_pRead = mz_zip_file_read_func;
+		archive.m_pIO_opaque = rw;
+		if (!mz_zip_reader_init(&archive, sz, 0)) goto fail;
+		localization.text = (char *) mz_zip_reader_extract_file_to_heap(&archive, file, &size, 0);
+		if (!localization.text) goto fail;
+		mz_zip_reader_end(&archive);
+		SDL_RWclose(rw);
+		localization.text = (char *) realloc(localization.text, size+1);
+		localization.text[size] = 0;
+	}
+	else
+	{
+		sz = SDL_RWsize(rw);
+		if (sz <= 0) goto fail;
+		localization.text = (char *) calloc(1, sz+1);
+		if (!localization.text)
+		{
+fail:			mz_zip_reader_end(&archive);
+			if (rw) SDL_RWclose(rw);
+			Con_Printf("Couldn't load '%s'\nfrom '%s'\n", file, com_basedir);
+			return;
+		}
+		SDL_RWread(rw, localization.text, 1, sz);
+		SDL_RWclose(rw);
+	}
+
+	cursor = localization.text;
+
+	// skip BOM
+	if ((unsigned char)(cursor[0]) == 0xEF && (unsigned char)(cursor[1]) == 0xBB && cursor[2] == 0xB)
+		cursor += 3;
+
+	lineno = 0;
+	while (*cursor)
+	{
+		char *line, *equals;
+
+		lineno++;
+
+		// skip leading whitespace
+		while (q_isblank(*cursor))
+			++cursor;
+
+		line = cursor;
+		equals = NULL;
+		// find line end and first equals sign, if any
+		while (*cursor && *cursor != '\n')
+		{
+			if (*cursor == '=' && !equals)
+				equals = cursor;
+			cursor++;
+		}
+
+		if (line[0] == '/')
+		{
+			if (line[1] != '/')
+				Con_DPrintf("LOC_LoadFile: malformed comment on line %d\n", lineno);
+		}
+		else if (equals)
+		{
+			char *key_end = equals;
+			qboolean leading_quote;
+			qboolean trailing_quote;
+			locentry_t *entry;
+			char *value_src;
+			char *value_dst;
+			char *value;
+
+			// trim whitespace before equals sign
+			while (key_end != line && q_isspace(key_end[-1]))
+				key_end--;
+			*key_end = 0;
+
+			value = equals + 1;
+			// skip whitespace after equals sign
+			while (value != cursor && q_isspace(*value))
+				value++;
+
+			leading_quote = (*value == '\"');
+			trailing_quote = false;
+			value += leading_quote;
+
+			// transform escape sequences in-place
+			value_src = value;
+			value_dst = value;
+			while (value_src != cursor)
+			{
+				if (*value_src == '\\' && value_src + 1 != cursor)
+				{
+					char c = value_src[1];
+					value_src += 2;
+					switch (c)
+					{
+						case 'n': *value_dst++ = '\n'; break;
+						case 't': *value_dst++ = '\t'; break;
+						case 'v': *value_dst++ = '\v'; break;
+						case 'b': *value_dst++ = '\b'; break;
+						case 'f': *value_dst++ = '\f'; break;
+
+						case '"':
+						case '\'':
+							*value_dst++ = c;
+							break;
+
+						default:
+							Con_Printf("LOC_LoadFile: unrecognized escape sequence \\%c on line %d\n", c, lineno);
+							*value_dst++ = c;
+							break;
+					}
+					continue;
+				}
+
+				if (*value_src == '\"')
+				{
+					trailing_quote = true;
+					*value_dst = 0;
+					break;
+				}
+
+				*value_dst++ = *value_src++;
+			}
+
+			// if not a quoted string, trim trailing whitespace
+			if (!trailing_quote)
+			{
+				while (value_dst != value && q_isblank(value_dst[-1]))
+				{
+					*value_dst = 0;
+					value_dst--;
+				}
+			}
+
+			if (localization.numentries == localization.maxnumentries)
+			{
+				// grow by 50%
+				localization.maxnumentries += localization.maxnumentries >> 1;
+				localization.maxnumentries = q_max(localization.maxnumentries, 32);
+				localization.entries = (locentry_t*) realloc(localization.entries, sizeof(*localization.entries) * localization.maxnumentries);
+			}
+
+			entry = &localization.entries[localization.numentries++];
+			entry->key = line;
+			entry->value = value;
+		}
+
+		if (*cursor)
+			*cursor++ = 0; // terminate line and advance to next
+	}
+
+	// hash all entries
+
+	localization.numindices = localization.numentries * 2; // 50% load factor
+	if (localization.numindices == 0)
+	{
+		Con_Printf("No localized strings in file '%s'\n", file);
+		return;
+	}
+
+	localization.indices = (unsigned*) realloc(localization.indices, localization.numindices * sizeof(*localization.indices));
+	memset(localization.indices, 0, localization.numindices * sizeof(*localization.indices));
+
+	for (i = 0; i < localization.numentries; i++)
+	{
+		locentry_t *entry = &localization.entries[i];
+		unsigned pos = COM_HashString(entry->key) % localization.numindices, end = pos;
+
+		for (;;)
+		{
+			if (!localization.indices[pos])
+			{
+				localization.indices[pos] = i + 1;
+				break;
+			}
+
+			++pos;
+			if (pos == localization.numindices)
+				pos = 0;
+
+			if (pos == end)
+				Sys_Error("LOC_LoadFile failed");
+		}
+	}
+
+	Con_Printf("Loaded %d strings from '%s'\n", localization.numentries, file);
+}
+
+/*
+================
+LOC_Init
+================
+*/
+void LOC_Init(void)
+{
+	LOC_LoadFile("localization/loc_english.txt");
+}
+
+/*
+================
+LOC_Shutdown
+================
+*/
+void LOC_Shutdown(void)
+{
+	free(localization.indices);
+	free(localization.entries);
+	free(localization.text);
+}
+
+/*
+================
+LOC_GetRawString
+
+Returns localized string if available, or NULL otherwise
+================
+*/
+const char* LOC_GetRawString (const char *key)
+{
+	unsigned pos, end;
+
+	if (!localization.numindices || !key || !*key || *key != '$')
+		return NULL;
+	key++;
+
+	pos = COM_HashString(key) % localization.numindices;
+	end = pos;
+
+	do
+	{
+		unsigned idx = localization.indices[pos];
+		locentry_t *entry;
+		if (!idx)
+			return NULL;
+
+		entry = &localization.entries[idx - 1];
+		if (!Q_strcmp(entry->key, key))
+			return entry->value;
+
+		++pos;
+		if (pos == localization.numindices)
+			pos = 0;
+	} while (pos != end);
+
+	return NULL;
+}
+
+/*
+================
+LOC_GetString
+
+Returns localized string if available, or input string otherwise
+================
+*/
+const char* LOC_GetString (const char *key)
+{
+	const char* value = LOC_GetRawString(key);
+	return value ? value : key;
+}
+
+/*
+================
+LOC_ParseArg
+
+Returns argument index (>= 0) and advances the string if it starts with a placeholder ({} or {N}),
+otherwise returns a negative value and leaves the pointer unchanged
+================
+*/
+static int LOC_ParseArg (const char **pstr)
+{
+	int arg;
+	const char *str = *pstr;
+
+	// opening brace
+	if (*str != '{')
+		return -1;
+	++str;
+
+	// optional index, defaulting to 0
+	arg = 0;
+	while (q_isdigit(*str))
+		arg = arg * 10 + *str++ - '0';
+
+	// closing brace
+	if (*str != '}')
+		return -1;
+	*pstr = ++str;
+
+	return arg;
+}
+
+/*
+================
+LOC_HasPlaceholders
+================
+*/
+qboolean LOC_HasPlaceholders (const char *str)
+{
+	if (!localization.numindices)
+		return false;
+	while (*str)
+	{
+		if (LOC_ParseArg(&str) >= 0)
+			return true;
+		str++;
+	}
+	return false;
+}
+
+/*
+================
+LOC_Format
+
+Replaces placeholders (of the form {} or {N}) with the corresponding arguments
+
+Returns number of written chars, excluding the NUL terminator
+If len > 0, output is always NUL-terminated
+================
+*/
+size_t LOC_Format (const char *format, const char* (*getarg_fn) (int idx, void* userdata), void* userdata, char* out, size_t len)
+{
+	size_t written = 0;
+	int numargs = 0;
+
+	if (!len)
+	{
+		Con_DPrintf("LOC_Format: no output space\n");
+		return 0;
+	}
+	--len; // reserve space for the terminator
+
+	while (*format && written < len)
+	{
+		const char* insert;
+		size_t space_left;
+		size_t insert_len;
+		int argindex = LOC_ParseArg(&format);
+
+		if (argindex < 0)
+		{
+			out[written++] = *format++;
+			continue;
+		}
+
+		insert = getarg_fn(argindex, userdata);
+		space_left = len - written;
+		insert_len = Q_strlen(insert);
+
+		if (insert_len > space_left)
+		{
+			Con_DPrintf("LOC_Format: overflow at argument #%d\n", numargs);
+			insert_len = space_left;
+		}
+
+		Q_memcpy(out + written, insert, insert_len);
+		written += insert_len;
+	}
+
+	if (*format)
+		Con_DPrintf("LOC_Format: overflow\n");
+
+	out[written] = 0;
+
+	return written;
+}
