@@ -272,48 +272,21 @@ qboolean R_CullBox (vec3_t emins, vec3_t emaxs)
 {
 	int i;
 	mplane_t *p;
+	byte signbits;
+	float vec[3];
 	for (i = 0;i < 4;i++)
 	{
 		p = frustum + i;
-		switch(p->signbits)
-		{
-		default:
-		case 0:
-			if (p->normal[0]*emaxs[0] + p->normal[1]*emaxs[1] + p->normal[2]*emaxs[2] < p->dist)
-				return true;
-			break;
-		case 1:
-			if (p->normal[0]*emins[0] + p->normal[1]*emaxs[1] + p->normal[2]*emaxs[2] < p->dist)
-				return true;
-			break;
-		case 2:
-			if (p->normal[0]*emaxs[0] + p->normal[1]*emins[1] + p->normal[2]*emaxs[2] < p->dist)
-				return true;
-			break;
-		case 3:
-			if (p->normal[0]*emins[0] + p->normal[1]*emins[1] + p->normal[2]*emaxs[2] < p->dist)
-				return true;
-			break;
-		case 4:
-			if (p->normal[0]*emaxs[0] + p->normal[1]*emaxs[1] + p->normal[2]*emins[2] < p->dist)
-				return true;
-			break;
-		case 5:
-			if (p->normal[0]*emins[0] + p->normal[1]*emaxs[1] + p->normal[2]*emins[2] < p->dist)
-				return true;
-			break;
-		case 6:
-			if (p->normal[0]*emaxs[0] + p->normal[1]*emins[1] + p->normal[2]*emins[2] < p->dist)
-				return true;
-			break;
-		case 7:
-			if (p->normal[0]*emins[0] + p->normal[1]*emins[1] + p->normal[2]*emins[2] < p->dist)
-				return true;
-			break;
-		}
+		signbits = p->signbits;
+		vec[0] = ((signbits % 2)<1) ? emaxs[0] : emins[0];
+		vec[1] = ((signbits % 4)<2) ? emaxs[1] : emins[1];
+		vec[2] = ((signbits % 8)<4) ? emaxs[2] : emins[2];
+		if (p->normal[0]*vec[0] + p->normal[1]*vec[1] + p->normal[2]*vec[2] < p->dist)
+			return true;
 	}
 	return false;
 }
+
 /*
 ===============
 R_CullModelForEntity -- johnfitz -- uses correct bounds based on rotation
@@ -538,9 +511,6 @@ R_SetupScene -- johnfitz -- this is the stuff that needs to be done once per eye
 */
 void R_SetupScene (void)
 {
-	R_PushDlights ();
-	R_AnimateLight ();
-	r_framecount++;
 	R_SetupGL ();
 }
 
@@ -551,6 +521,11 @@ R_SetupView -- johnfitz -- this is the stuff that needs to be done once per fram
 */
 void R_SetupView (void)
 {
+	// Need to do those early because we now update dynamic light maps during R_MarkSurfaces
+	R_PushDlights ();
+	R_AnimateLight ();
+	r_framecount++;
+
 	Fog_SetupFrame (); //johnfitz
 
 // build the transformation matrix for the given view angles
@@ -585,8 +560,6 @@ void R_SetupView (void)
 
 	R_MarkSurfaces (); //johnfitz -- create texture chains from PVS
 
-	R_CullSurfaces (); //johnfitz -- do after R_SetFrustum and R_MarkSurfaces
-
 	R_UpdateWarpTextures (); //johnfitz -- do this before R_Clear
 
 	R_Clear ();
