@@ -35,7 +35,7 @@ extern	int rs_skypasses; //for r_speeds readout
 float	skyflatcolor[3];
 float	skymins[2][6], skymaxs[2][6];
 
-char	skybox_name[32] = ""; //name of current skybox, or "" if no skybox
+char	skybox_name[1024]; //name of current skybox, or "" if no skybox
 
 gltexture_t	*skybox_textures[6];
 gltexture_t	*solidskytexture, *alphaskytexture;
@@ -63,8 +63,8 @@ int	st_to_vec[6][3] =
 	{-3,1,2},
 	{1,3,2},
 	{-1,-3,2},
- 	{-2,-1,3},		// straight up
- 	{2,-1,-3}		// straight down
+	{-2,-1,3},		// straight up
+	{2,-1,-3}		// straight down
 };
 
 int	vec_to_st[6][3] =
@@ -95,41 +95,38 @@ A sky texture is 256*128, with the left side being a masked overlay
 void Sky_LoadTexture (texture_t *mt)
 {
 	char		texturename[64];
-	int			i, j, p, r, g, b, count;
-	byte		*src;
-	static byte	front_data[128*128]; //FIXME: Hunk_Alloc
-	static byte	back_data[128*128]; //FIXME: Hunk_Alloc
-	unsigned	*rgba;
+	unsigned	x, y, p, r, g, b, count, halfwidth, *rgba;
+	byte		*src, *front_data, *back_data;
 
-	src = (byte *)mt + mt->offsets[0];
+	if (mt->width != 256 || mt->height != 128)
+	{
+		Con_Warning ("Sky texture %s is %d x %d, expected 256 x 128\n", mt->name, mt->width, mt->height);
+		if (mt->width < 2 || mt->height < 1)
+			return;
+	}
+
+	halfwidth = mt->width / 2;
+	back_data = (byte *) Hunk_AllocName (halfwidth*mt->height*2, "skytex");
+	front_data = back_data + halfwidth*mt->height;
+	src = (byte *)(mt + 1);
 
 // extract back layer and upload
-	for (i=0 ; i<128 ; i++)
-		for (j=0 ; j<128 ; j++)
-			back_data[(i*128) + j] = src[i*256 + j + 128];
+	for (y=0 ; y<mt->height ; y++)
+		memcpy (back_data + y*halfwidth, src + halfwidth + y*mt->width, halfwidth);
 
 	q_snprintf(texturename, sizeof(texturename), "%s:%s_back", loadmodel->name, mt->name);
-	solidskytexture = TexMgr_LoadImage (loadmodel, texturename, 128, 128, SRC_INDEXED, back_data, "", (src_offset_t)back_data, TEXPREF_NONE);
+	solidskytexture = TexMgr_LoadImage (loadmodel, texturename, halfwidth, mt->height, SRC_INDEXED, back_data, "", (src_offset_t)back_data, TEXPREF_NONE);
 
 // extract front layer and upload
-	for (i=0 ; i<128 ; i++)
-		for (j=0 ; j<128 ; j++)
-		{
-			front_data[(i*128) + j] = src[i*256 + j];
-			if (front_data[(i*128) + j] == 0)
-				front_data[(i*128) + j] = 255;
-		}
-
-	q_snprintf(texturename, sizeof(texturename), "%s:%s_front", loadmodel->name, mt->name);
-	alphaskytexture = TexMgr_LoadImage (loadmodel, texturename, 128, 128, SRC_INDEXED, front_data, "", (src_offset_t)front_data, TEXPREF_ALPHA);
-
-// calculate r_fastsky color based on average of all opaque foreground colors
 	r = g = b = count = 0;
-	for (i=0 ; i<128 ; i++)
-		for (j=0 ; j<128 ; j++)
+	for (y=0 ; y<mt->height ; src+=mt->width, front_data+=halfwidth, y++)
+	{
+		for (x=0 ; x<halfwidth ; x++)
 		{
-			p = src[i*256 + j];
-			if (p != 0)
+			p = src[x];
+			if (p == 0)
+				p = 255;
+			else
 			{
 				rgba = &d_8to24table[p];
 				r += ((byte *)rgba)[0];
@@ -137,7 +134,75 @@ void Sky_LoadTexture (texture_t *mt)
 				b += ((byte *)rgba)[2];
 				count++;
 			}
+			front_data[x] = p;
 		}
+	}
+
+	front_data = back_data + halfwidth*mt->height;
+	q_snprintf(texturename, sizeof(texturename), "%s:%s_front", loadmodel->name, mt->name);
+	alphaskytexture = TexMgr_LoadImage (loadmodel, texturename, halfwidth, mt->height, SRC_INDEXED, front_data, "", (src_offset_t)front_data, TEXPREF_ALPHA);
+
+// calculate r_fastsky color based on average of all opaque foreground colors
+	skyflatcolor[0] = (float)r/(count*255);
+	skyflatcolor[1] = (float)g/(count*255);
+	skyflatcolor[2] = (float)b/(count*255);
+}
+
+/*
+=============
+Sky_LoadTextureQ64
+
+Quake64 sky textures are 32*64
+==============
+*/
+void Sky_LoadTextureQ64 (texture_t *mt)
+{
+	char		texturename[64];
+	unsigned	i, p, r, g, b, count, halfheight, *rgba;
+	byte		*front, *back, *front_rgba;
+
+	if (mt->width != 32 || mt->height != 64)
+	{
+		Con_DWarning ("Q64 sky texture %s is %d x %d, expected 32 x 64\n", mt->name, mt->width, mt->height);
+		if (mt->width < 1 || mt->height < 2)
+			return;
+	}
+
+	// pointers to both layer textures
+	halfheight = mt->height / 2;
+	front = (byte *)(mt+1);
+	back = (byte *)(mt+1) + mt->width*halfheight;
+	front_rgba = (byte *) Hunk_AllocName (4*mt->width*halfheight, "q64_skytex");
+
+	// Normal indexed texture for the back layer
+	q_snprintf(texturename, sizeof(texturename), "%s:%s_back", loadmodel->name, mt->name);
+	solidskytexture = TexMgr_LoadImage (loadmodel, texturename, mt->width, halfheight, SRC_INDEXED, back, "", (src_offset_t)back, TEXPREF_NONE);
+
+	// front layer, convert to RGBA and upload
+	p = r = g = b = count = 0;
+
+	for (i=mt->width*halfheight ; i!=0 ; i--)
+	{
+		rgba = &d_8to24table[*front++];
+
+		// RGB
+		front_rgba[p++] = ((byte*)rgba)[0];
+		front_rgba[p++] = ((byte*)rgba)[1];
+		front_rgba[p++] = ((byte*)rgba)[2];
+		// Alpha
+		front_rgba[p++] = 128; // this look ok to me!
+
+		// Fast sky
+		r += ((byte *)rgba)[0];
+		g += ((byte *)rgba)[1];
+		b += ((byte *)rgba)[2];
+		count++;
+	}
+
+	q_snprintf(texturename, sizeof(texturename), "%s:%s_front", loadmodel->name, mt->name);
+	alphaskytexture = TexMgr_LoadImage (loadmodel, texturename, mt->width, halfheight, SRC_RGBA, front_rgba, "", (src_offset_t)front_rgba, TEXPREF_ALPHA);
+
+	// calculate r_fastsky color based on average of all opaque foreground colors
 	skyflatcolor[0] = (float)r/(count*255);
 	skyflatcolor[1] = (float)g/(count*255);
 	skyflatcolor[2] = (float)b/(count*255);
@@ -205,7 +270,25 @@ void Sky_LoadSkyBox (const char *name)
 		return;
 	}
 
-	strcpy(skybox_name, name);
+	q_strlcpy(skybox_name, name, sizeof(skybox_name));
+}
+
+/*
+=================
+Sky_ClearAll
+
+Called on map unload/game change to avoid keeping pointers to freed data
+=================
+*/
+void Sky_ClearAll (void)
+{
+	int i;
+
+	skybox_name[0] = 0;
+	for (i=0; i<6; i++)
+		skybox_textures[i] = NULL;
+	solidskytexture = NULL;
+	alphaskytexture = NULL;
 }
 
 /*
@@ -217,14 +300,7 @@ void Sky_NewMap (void)
 {
 	char	key[128], value[4096];
 	const char	*data;
-	int		i;
 
-	//
-	// initially no sky
-	//
-	skybox_name[0] = 0;
-	for (i=0; i<6; i++)
-		skybox_textures[i] = NULL;
 	skyfog = r_skyfog.value;
 
 	//
@@ -264,7 +340,7 @@ void Sky_NewMap (void)
 		if (!strcmp("skyfog", key))
 			skyfog = atof(value);
 
-#if 1 //also accept non-standard keys
+#if 1 /* also accept non-standard keys */
 		else if (!strcmp("skyname", key)) //half-life
 			Sky_LoadSkyBox(value);
 		else if (!strcmp("qlsky", key)) //quake lives
@@ -321,6 +397,7 @@ void Sky_Init (void)
 
 	Cmd_AddCommand ("sky",Sky_SkyCommand_f);
 
+	skybox_name[0] = 0;
 	for (i=0; i<6; i++)
 		skybox_textures[i] = NULL;
 }
@@ -549,8 +626,7 @@ void Sky_ProcessTextureChains (void)
 			continue;
 
 		for (s = t->texturechains[chain_world]; s; s = s->texturechain)
-			if (!s->culled)
-				Sky_ProcessPoly (s->polys);
+			Sky_ProcessPoly (s->polys);
 	}
 }
 
@@ -692,7 +768,7 @@ FIXME: eliminate cracks by adding an extra vert on tjuncs
 */
 void Sky_DrawSkyBox (void)
 {
-	int		i;
+	int i;
 
 	for (i=0 ; i<6 ; i++)
 	{
@@ -701,7 +777,7 @@ void Sky_DrawSkyBox (void)
 
 		GL_Bind (skybox_textures[skytexorder[i]]);
 
-#if 1 //FIXME: this is to avoid tjunctions until i can do it the right way
+#if 1 /* FIXME: this is to avoid tjunctions until i can do it the right way */
 		skymins[0][i] = -1;
 		skymins[1][i] = -1;
 		skymaxs[0][i] = 1;
@@ -902,7 +978,7 @@ void Sky_DrawFace (int axis)
 	vec3_t		verts[4];
 	int			i, j, start;
 	float		di,qi,dj,qj;
-	vec3_t		vup, vright, temp, temp2;
+	vec3_t		up, right, temp, temp2;
 
 	Sky_SetBoxVert(-1.0, -1.0, axis, verts[0]);
 	Sky_SetBoxVert(-1.0,  1.0, axis, verts[1]);
@@ -912,8 +988,8 @@ void Sky_DrawFace (int axis)
 	start = Hunk_LowMark ();
 	p = (glpoly_t *) Hunk_Alloc(sizeof(glpoly_t));
 
-	VectorSubtract(verts[2],verts[3],vup);
-	VectorSubtract(verts[2],verts[1],vright);
+	VectorSubtract(verts[2],verts[3],up);
+	VectorSubtract(verts[2],verts[1],right);
 
 	di = q_max((int)r_sky_quality.value, 1);
 	qi = 1.0 / di;
@@ -929,15 +1005,15 @@ void Sky_DrawFace (int axis)
 				continue;
 
 			//if (i&1 ^ j&1) continue; //checkerboard test
-			VectorScale (vright, qi*i, temp);
-			VectorScale (vup, qj*j, temp2);
+			VectorScale (right, qi*i, temp);
+			VectorScale (up, qj*j, temp2);
 			VectorAdd(temp,temp2,temp);
 			VectorAdd(verts[0],temp,p->verts[0]);
 
-			VectorScale (vup, qj, temp);
+			VectorScale (up, qj, temp);
 			VectorAdd (p->verts[0],temp,p->verts[1]);
 
-			VectorScale (vright, qi, temp);
+			VectorScale (right, qi, temp);
 			VectorAdd (p->verts[1],temp,p->verts[2]);
 
 			VectorAdd (p->verts[0],temp,p->verts[3]);
@@ -979,10 +1055,10 @@ called once per frame before drawing anything else
 */
 void Sky_DrawSky (void)
 {
-	int				i;
+	int i;
 
 	//in these special render modes, the sky faces are handled in the normal world/brush renderer
-	if (r_drawflat_cheatsafe || r_lightmap_cheatsafe )
+	if (r_drawflat_cheatsafe || r_lightmap_cheatsafe)
 		return;
 
 	//
@@ -990,8 +1066,8 @@ void Sky_DrawSky (void)
 	//
 	for (i=0 ; i<6 ; i++)
 	{
-		skymins[0][i] = skymins[1][i] = 9999;
-		skymaxs[0][i] = skymaxs[1][i] = -9999;
+		skymins[0][i] = skymins[1][i] = FLT_MAX;
+		skymaxs[0][i] = skymaxs[1][i] = -FLT_MAX;
 	}
 
 	//
@@ -1019,7 +1095,7 @@ void Sky_DrawSky (void)
 		if (skybox_name[0])
 			Sky_DrawSkyBox ();
 		else
-			Sky_DrawSkyLayers();
+			Sky_DrawSkyLayers ();
 
 		glDepthMask(1);
 		glDepthFunc(GL_LEQUAL);
