@@ -33,7 +33,7 @@ static cvar_t	gl_max_size = {"gl_max_size", "0", CVAR_NONE};
 static cvar_t	gl_picmip = {"gl_picmip", "0", CVAR_NONE};
 static GLint	gl_hardware_maxsize;
 
-#define	MAX_GLTEXTURES	2048
+#define	MAX_GLTEXTURES	4096
 static int numgltextures;
 static gltexture_t	*active_gltextures, *free_gltextures;
 gltexture_t		*notexture, *nulltexture;
@@ -1282,34 +1282,32 @@ void TexMgr_ReloadImage (gltexture_t *glt, int shirt, int pants)
 //
 	mark = Hunk_LowMark ();
 
-	if (glt->source_file[0] && glt->source_offset)
-	{
+	if (glt->source_file[0] && glt->source_offset) {
 		//lump inside file
-		long size;
 		FILE *f;
 		COM_FOpenFile(glt->source_file, &f, NULL);
-		if (!f)
-			goto invalid;
+		if (!f) goto invalid;
 		fseek (f, glt->source_offset, SEEK_CUR);
-		size = (long) (glt->source_width * glt->source_height);
+		size = glt->source_width * glt->source_height;
 		/* should be SRC_INDEXED, but no harm being paranoid:  */
-		if (glt->source_format == SRC_RGBA)
+		if (glt->source_format == SRC_RGBA) {
 			size *= 4;
-		else if (glt->source_format == SRC_LIGHTMAP)
+		}
+		else if (glt->source_format == SRC_LIGHTMAP) {
 			size *= lightmap_bytes;
+		}
 		data = (byte *) Hunk_Alloc (size);
 		fread (data, 1, size, f);
 		fclose (f);
 	}
-	else if (glt->source_file[0] && !glt->source_offset)
+	else if (glt->source_file[0] && !glt->source_offset) {
 		data = Image_LoadImage (glt->source_file, (int *)&glt->source_width, (int *)&glt->source_height); //simple file
-	else if (!glt->source_file[0] && glt->source_offset)
+	}
+	else if (!glt->source_file[0] && glt->source_offset) {
 		data = (byte *) glt->source_offset; //image in memory
-
-	if (!data)
-	{
-invalid:
-		Con_Printf ("TexMgr_ReloadImage: invalid source for %s\n", glt->name);
+	}
+	if (!data) {
+invalid:	Con_Printf ("TexMgr_ReloadImage: invalid source for %s\n", glt->name);
 		Hunk_FreeToLowMark(mark);
 		return;
 	}
