@@ -2532,6 +2532,13 @@ void M_ServerList_Key (int k)
 
 }
 
+//=============================================================================
+/* Credits menu -- used by the 2021 re-release */
+
+void M_Menu_Credits_f (void)
+{
+}
+
 //=============================================================================
 /* Menu Subsystem */
 
@@ -2551,6 +2558,7 @@ void M_Init (void)
 	Cmd_AddCommand ("menu_video", M_Menu_Video_f);
 	Cmd_AddCommand ("help", M_Menu_Help_f);
 	Cmd_AddCommand ("menu_quit", M_Menu_Quit_f);
+	Cmd_AddCommand ("menu_credits", M_Menu_Credits_f); // needed by the 2021 re-release
 }
 
 
