@@ -25,6 +25,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "quakedef.h"
 
 extern cvar_t r_drawflat, gl_overbright_models, gl_fullbrights, r_lerpmodels, r_lerpmove; //johnfitz
+extern cvar_t scr_fov, cl_gun_fovscale;
 
 //up to 16 color translated skins
 gltexture_t *playertextures[MAX_SCOREBOARD]; //johnfitz -- changed to an array of pointers
@@ -546,8 +547,14 @@ void R_SetupAliasLighting (entity_t	*e)
 	int			i;
 	int		quantizedangle;
 	float		radiansangle;
+	vec3_t		lpos;
 
-	R_LightPoint (e->origin);
+	VectorCopy (e->origin, lpos);
+	// start the light trace from slightly above the origin
+	// this helps with models whose origin is below ground level, but are otherwise visible
+	// (e.g. some of the candles in the DOTM start map, which would otherwise appear black)
+	lpos[2] += e->model->maxs[2] * 0.5f;
+	R_LightPoint (lpos);
 
 	//add dlights
 	for (i=0 ; i<MAX_DLIGHTS ; i++)
@@ -629,6 +636,7 @@ void R_DrawAliasModel (entity_t *e)
 	gltexture_t	*tx, *fb;
 	lerpdata_t	lerpdata;
 	qboolean	alphatest = !!(e->model->flags & MF_HOLEY);
+	float		fovscale = 1.0f;
 
 	//
 	// setup pose/lerp data -- do it first so we don't miss updates due to culling
@@ -646,10 +654,13 @@ void R_DrawAliasModel (entity_t *e)
 	//
 	// transform it
 	//
+	if (e == &cl.viewent && scr_fov.value > 90.f && cl_gun_fovscale.value)
+		fovscale = tan(scr_fov.value * (0.5f * M_PI / 180.f));
+
 	glPushMatrix ();
 	R_RotateForEntity (lerpdata.origin, lerpdata.angles);
-	glTranslatef (paliashdr->scale_origin[0], paliashdr->scale_origin[1], paliashdr->scale_origin[2]);
-	glScalef (paliashdr->scale[0], paliashdr->scale[1], paliashdr->scale[2]);
+	glTranslatef (paliashdr->scale_origin[0], paliashdr->scale_origin[1] * fovscale, paliashdr->scale_origin[2] * fovscale);
+	glScalef (paliashdr->scale[0], paliashdr->scale[1] * fovscale, paliashdr->scale[2] * fovscale);
 
 	//
 	// random stuff
