@@ -31,7 +31,7 @@ int		gl_lightmap_format;
 int		lightmap_bytes;
 
 #define MAX_SANITY_LIGHTMAPS (1u<<20)
-struct lightmap_s	*lightmap;
+struct lightmap_s	*lightmaps;
 int					lightmap_count;
 int					last_lightmap_allocated;
 int					allocated[LMBLOCK_WIDTH];
@@ -567,6 +567,7 @@ void R_DrawBrushModel (entity_t *e)
 			(!(psurf->flags & SURF_PLANEBACK) && (dot > BACKFACE_EPSILON)))
 		{
 			R_ChainSurface (psurf, chain_model);
+			R_RenderDynamicLightmaps(psurf);
 			rs_brushpolys++;
 		}
 	}
@@ -663,8 +664,8 @@ void R_RenderDynamicLightmaps (msurface_t *fa)
 		return;
 
 	// add to lightmap chain
-	fa->polys->chain = lightmap[fa->lightmaptexturenum].polys;
-	lightmap[fa->lightmaptexturenum].polys = fa->polys;
+	fa->polys->chain = lightmaps[fa->lightmaptexturenum].polys;
+	lightmaps[fa->lightmaptexturenum].polys = fa->polys;
 
 	// check for lightmap modification
 	for (maps=0; maps < MAXLIGHTMAPS && fa->styles[maps] != 255; maps++)
@@ -677,7 +678,7 @@ void R_RenderDynamicLightmaps (msurface_t *fa)
 dynamic:
 		if (r_dynamic.value)
 		{
-			struct lightmap_s *lm = &lightmap[fa->lightmaptexturenum];
+			struct lightmap_s *lm = &lightmaps[fa->lightmaptexturenum];
 			lm->modified = true;
 			theRect = &lm->rectchange;
 			if (fa->light_t < theRect->t) {
@@ -724,11 +725,9 @@ int AllocBlock (int w, int h, int *x, int *y)
 		if (texnum == lightmap_count)
 		{
 			lightmap_count++;
-			lightmap = (struct lightmap_s *) realloc(lightmap, sizeof(*lightmap)*lightmap_count);
-			memset(&lightmap[texnum], 0, sizeof(lightmap[texnum]));
-			/* FIXME: we leave 'gaps' in malloc()ed data,  CRC_Block() later accesses
-			 * that uninitialized data and valgrind complains for it.  use calloc() ? */
-			lightmap[texnum].data = (byte *) malloc(4*LMBLOCK_WIDTH*LMBLOCK_HEIGHT);
+			lightmaps = (struct lightmap_s *) realloc(lightmaps, sizeof(*lightmaps)*lightmap_count);
+			memset(&lightmaps[texnum], 0, sizeof(lightmaps[texnum]));
+			lightmaps[texnum].data = (byte *) calloc(1, 4*LMBLOCK_WIDTH*LMBLOCK_HEIGHT);
 			//as we're only tracking one texture, we don't need multiple copies of allocated any more.
 			memset(allocated, 0, sizeof(allocated));
 		}
@@ -786,7 +785,7 @@ void GL_CreateSurfaceLightmap (msurface_t *surf)
 	tmax = (surf->extents[1]>>4)+1;
 
 	surf->lightmaptexturenum = AllocBlock (smax, tmax, &surf->light_s, &surf->light_t);
-	base = lightmap[surf->lightmaptexturenum].data;
+	base = lightmaps[surf->lightmaptexturenum].data;
 	base += (surf->light_t * LMBLOCK_WIDTH + surf->light_s) * lightmap_bytes;
 	R_BuildLightMap (surf, base, LMBLOCK_WIDTH*lightmap_bytes);
 }
@@ -840,6 +839,13 @@ void BuildSurfaceDisplayList (msurface_t *fa)
 		poly->verts[i][3] = s;
 		poly->verts[i][4] = t;
 
+		// Q64 RERELEASE texture shift
+		if (fa->texinfo->texture->shift > 0)
+		{
+			poly->verts[i][3] /= ( 2 * fa->texinfo->texture->shift);
+			poly->verts[i][4] /= ( 2 * fa->texinfo->texture->shift);
+		}
+
 		//
 		// lightmap texture coordinates
 		//
@@ -883,9 +889,9 @@ void GL_BuildLightmaps (void)
 
 	//Spike -- wipe out all the lightmap data (johnfitz -- the gltexture objects were already freed by Mod_ClearAll)
 	for (i=0; i < lightmap_count; i++)
-		free(lightmap[i].data);
-	free(lightmap);
-	lightmap = NULL;
+		free(lightmaps[i].data);
+	free(lightmaps);
+	lightmaps = NULL;
 	last_lightmap_allocated = 0;
 	lightmap_count = 0;
 
@@ -928,7 +934,7 @@ void GL_BuildLightmaps (void)
 	//
 	for (i=0; i<lightmap_count; i++)
 	{
-		lm = &lightmap[i];
+		lm = &lightmaps[i];
 		lm->modified = false;
 		lm->rectchange.l = LMBLOCK_WIDTH;
 		lm->rectchange.t = LMBLOCK_HEIGHT;
@@ -1252,7 +1258,7 @@ assumes lightmap texture is already bound
 */
 static void R_UploadLightmap(int lmap)
 {
-	struct lightmap_s *lm = &lightmap[lmap];
+	struct lightmap_s *lm = &lightmaps[lmap];
 
 	if (!lm->modified)
 		return;
@@ -1275,10 +1281,10 @@ void R_UploadLightmaps (void)
 
 	for (lmap = 0; lmap < lightmap_count; lmap++)
 	{
-		if (!lightmap[lmap].modified)
+		if (!lightmaps[lmap].modified)
 			continue;
 
-		GL_Bind (lightmap[lmap].texture);
+		GL_Bind (lightmaps[lmap].texture);
 		R_UploadLightmap(lmap);
 	}
 }
@@ -1308,7 +1314,7 @@ void R_RebuildAllLightmaps (void)
 		{
 			if (fa->flags & SURF_DRAWTILED)
 				continue;
-			base = lightmap[fa->lightmaptexturenum].data;
+			base = lightmaps[fa->lightmaptexturenum].data;
 			base += fa->light_t * LMBLOCK_WIDTH * lightmap_bytes + fa->light_s * lightmap_bytes;
 			R_BuildLightMap (fa, base, LMBLOCK_WIDTH*lightmap_bytes);
 		}
@@ -1317,8 +1323,8 @@ void R_RebuildAllLightmaps (void)
 	//for each lightmap, upload it
 	for (i=0; i<lightmap_count; i++)
 	{
-		GL_Bind (lightmap[i].texture);
+		GL_Bind (lightmaps[i].texture);
 		glTexSubImage2D (GL_TEXTURE_2D, 0, 0, 0, LMBLOCK_WIDTH, LMBLOCK_HEIGHT, gl_lightmap_format,
-				 GL_UNSIGNED_BYTE, lightmap[i].data);
+				 GL_UNSIGNED_BYTE, lightmaps[i].data);
 	}
 }
