@@ -284,12 +284,12 @@ void Z_Print (memzone_t *zone)
 
 //============================================================================
 
-#define	HUNK_SENTINAL	0x1df001ed
+#define	HUNK_SENTINEL	0x1df001ed
 
 #define HUNKNAME_LEN	24
 typedef struct
 {
-	int		sentinal;
+	int		sentinel;
 	int		size;		// including sizeof(hunk_t), -1 = not allocated
 	char	name[HUNKNAME_LEN];
 } hunk_t;
@@ -307,7 +307,7 @@ int		hunk_tempmark;
 ==============
 Hunk_Check
 
-Run consistancy and sentinal trahing checks
+Run consistancy and sentinel trahing checks
 ==============
 */
 void Hunk_Check (void)
@@ -316,8 +316,8 @@ void Hunk_Check (void)
 
 	for (h = (hunk_t *)hunk_base ; (byte *)h != hunk_base + hunk_low_used ; )
 	{
-		if (h->sentinal != HUNK_SENTINAL)
-			Sys_Error ("Hunk_Check: trahsed sentinal");
+		if (h->sentinel != HUNK_SENTINEL)
+			Sys_Error ("Hunk_Check: trashed sentinel");
 		if (h->size < (int) sizeof(hunk_t) || h->size + (byte *)h - hunk_base > hunk_size)
 			Sys_Error ("Hunk_Check: bad size");
 		h = (hunk_t *)((byte *)h+h->size);
@@ -373,8 +373,8 @@ void Hunk_Print (qboolean all)
 	//
 	// run consistancy checks
 	//
-		if (h->sentinal != HUNK_SENTINAL)
-			Sys_Error ("Hunk_Check: trahsed sentinal");
+		if (h->sentinel != HUNK_SENTINEL)
+			Sys_Error ("Hunk_Check: trashed sentinel");
 		if (h->size < (int) sizeof(hunk_t) || h->size + (byte *)h - hunk_base > hunk_size)
 			Sys_Error ("Hunk_Check: bad size");
 
@@ -449,7 +449,7 @@ void *Hunk_AllocName (int size, const char *name)
 	memset (h, 0, size);
 
 	h->size = size;
-	h->sentinal = HUNK_SENTINAL;
+	h->sentinel = HUNK_SENTINEL;
 	q_strlcpy (h->name, name, HUNKNAME_LEN);
 
 	return (void *)(h+1);
@@ -540,7 +540,7 @@ void *Hunk_HighAllocName (int size, const char *name)
 
 	memset (h, 0, size);
 	h->size = size;
-	h->sentinal = HUNK_SENTINAL;
+	h->sentinel = HUNK_SENTINEL;
 	q_strlcpy (h->name, name, HUNKNAME_LEN);
 
 	return (void *)(h+1);
@@ -906,7 +906,7 @@ void *Cache_Alloc (cache_user_t *c, int size, const char *name)
 	cache_system_t	*cs;
 
 	if (c->data)
-		Sys_Error ("Cache_Alloc: allready allocated");
+		Sys_Error ("Cache_Alloc: already allocated");
 
 	if (size <= 0)
 		Sys_Error ("Cache_Alloc: size %i", size);
